﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.CodeDom.Compiler;
using System.Reflection;
using System.IO;

namespace howto_load_runtime_enum
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Load and compile an enum file.
        private void mnuFileOpenEnumFile_Click(object sender, EventArgs e)
        {
            if (ofdEnum.ShowDialog() != DialogResult.OK) return;

            trvEnums.Nodes.Clear();

            // Use the C# DOM provider.
            CodeDomProvider code_provider =
                CodeDomProvider.CreateProvider("C#");

            // Generate a non-executable assembly in memory.
            CompilerParameters parameters = new CompilerParameters();
            parameters.GenerateInMemory = true;
            parameters.GenerateExecutable = false;

            // Get information about this assembly.
            // (So the enum can use attributes defined
            // in Attributes.cs.)
            string exe_name = Assembly.GetEntryAssembly().Location;
            parameters.ReferencedAssemblies.Add(exe_name);

            // Load the enum file's text placed
            // inside our namespace.
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("namespace howto_load_runtime_enum");
            sb.AppendLine("{");
            sb.AppendLine(File.ReadAllText(ofdEnum.FileName));
            sb.AppendLine("}");

            // Compile the code.
            CompilerResults results =
                code_provider.CompileAssemblyFromSource(parameters,
                    sb.ToString());

            // If there are errors, display them.
            if (results.Errors.Count > 0)
            {
                trvEnums.Nodes.Add("Error compiling the expression.");
                foreach (CompilerError compiler_error in results.Errors)
                {
                    trvEnums.Nodes.Add(compiler_error.ErrorText);
                }
                return;
            }

            // See what enumerations there are.
            foreach (Type type in results.CompiledAssembly.GetTypes())
            {
                // Only consider types that are enums.
                if (type.IsEnum)
                {
                    // Add a TreeView root node for the enum.
                    TreeNode enum_node = trvEnums.Nodes.Add(type.Name);

                    // Enumerate the Enum's fields (values).
                    FieldInfo[] enum_value_infos = type.GetFields();

                    // Loop over the enum's values.
                    foreach (FieldInfo value_info in enum_value_infos)
                    {
                        // See if this is a literal value (set at compile time).
                        if (value_info.IsLiteral)
                        {
                            // Add it to the Enum's root node.
                            TreeNode value_node =
                                enum_node.Nodes.Add(
                                    value_info.Name + " = " +
                                    ((int)value_info.GetValue(null)).ToString());

                            // Loop through the value's attributes.
                            foreach (Attribute attr in value_info.GetCustomAttributes(false))
                            {
                                // Start with the attribute's name.
                                string attr_text = attr.GetType().Name + "(";

                                // Add the attribute's fields.
                                foreach (FieldInfo field_info in attr.GetType().GetFields())
                                {
                                    attr_text +=
                                        field_info.Name + " = " +
                                        field_info.GetValue(attr).ToString() +
                                        ", ";
                                }

                                // If we have fields, remove the trailing ", ".
                                if (attr_text.Length > 0)
                                    attr_text = attr_text.Substring(0, attr_text.Length - 2);
                                attr_text += ")";

                                // Add this to the TreeView.
                                value_node.Nodes.Add(attr_text);
                            }
                        }
                    }
                }
            }
            trvEnums.ExpandAll();
        }

        private void mnuFileExit_Click(object sender, EventArgs e)
        {
            Close();
        }
    }
}
